using System;
using System.IO;
using System.Collections;
using System.Threading;
using System.Diagnostics;
using System.Reflection;

namespace Server
{
	/// <summary>
	/// Performs case-insensitive string operations.
	/// </summary>
	public class Insensitive
	{
		private static IComparer m_Comparer = System.Collections.CaseInsensitiveComparer.Default;

		public static IComparer Comparer
		{
			get 
			{
				return Insensitive.m_Comparer;
			}
		}

		public static int Compare(string a, string b)
		{
			return Insensitive.m_Comparer.Compare(a,b);
		}

		public static bool Contains(string a, string b)
		{
			if (((a==null) || (b==null)) || (a.Length < b.Length))
			{
				return false;
			}
			a = a.ToLower();
			b = b.ToLower();
			return !(a.IndexOf(b)<0);
		}

		public static bool EndsWith(string a, string b)
		{
			if (((a==null) || (b==null)) || (a.Length < b.Length))
			{
				return false;
			}
			return (Insensitive.m_Comparer.Compare(a.Substring((a.Length - b.Length)), b) == 0);
		}

		public static bool Equals(string a, string b)
		{
			if ((a==null) && (b==null))
			{
				return true;
			}
			if ((a==null) || ((b==null) || (a.Length != b.Length)))
			{
				return false;
			}
			return (Insensitive.m_Comparer.Compare(a,b)==0);
		}

		public static bool StartsWith(string a, string b)
		{
			if (((a==null) || (b==null)) || (a.Length < b.Length))
			{
				return false;
			}
			return (Insensitive.m_Comparer.Compare(a.Substring(0,b.Length), b) == 0);
		}
	}

	public class MultiTextWriter : TextWriter
	{
		private ArrayList m_Streams;
		public override System.Text.Encoding Encoding
		{
			get
			{
				return Encoding.Default;
			}
		}

		public MultiTextWriter(params TextWriter[] streams)
		{
			this.m_Streams = new ArrayList(streams);
			if (this.m_Streams.Count < 0)
			{
				throw new ArgumentException("You must specify at least one stream.");
			}
		} 

		public void Add(TextWriter tw)
		{
			this.m_Streams.Add(tw);
 		} 

		public void Remove(TextWriter tw)
		{
			this.m_Streams.Remove(tw);
 		} 

		public override void Write(char ch)
		{
			for (int streamNum = 0; (streamNum < this.m_Streams.Count); streamNum++)
			{
				((TextWriter) this.m_Streams[streamNum]).Write(ch);
			}
		} 

		public override void WriteLine(string line)
		{
			for (int streamNum = 0; (streamNum < this.m_Streams.Count); streamNum++)
			{
				((TextWriter) this.m_Streams[streamNum]).WriteLine(line);
			}
		} 

		public override void WriteLine(string line, params object[] args)
		{
			/*base.*/WriteLine(string.Format(line, args));
		} 
	}

	/// <summary>
	/// The core server class.
	/// </summary>
	public class Core
	{
		private static bool m_Service;
		private static bool m_Profiling;
		private static DateTime m_ProfileStart;
		private static TimeSpan m_ProfileTime;

		private static Thread m_Thread;
		private static Process m_Process;
		private static Assembly m_Assembly;
		private static string m_ExePath;
		private static string m_BaseDirectory;

		private static Thread timerThread;

		public static string ExePath
		{
			get
			{
				if (Core.m_ExePath == null)
				{
					Core.m_ExePath = Process.GetCurrentProcess().MainModule.FileName;
 				}
				return Core.m_ExePath; 
			}
		}

		public static string BaseDirectory
		{
			get
			{
				if (Core.m_BaseDirectory == null)
				{
					try
					{
						Core.m_BaseDirectory = Core.ExePath;
						if (Core.m_BaseDirectory.Length > 0)
						{
							Core.m_BaseDirectory = Path.GetDirectoryName(Core.m_BaseDirectory);
						}
					}
					catch (Exception ignored)
					{
						Core.m_BaseDirectory = "";
					}
				}
				return Core.m_BaseDirectory; 
			}
		}

		public static bool Profiling 
		{
			get 
			{
				return Core.m_Profiling;
			}
			set 
			{
				if (Core.m_Profiling == value)
				{
					return; 
				}
				Core.m_Profiling = value;
				if (DateTime.op_GreaterThan(Core.m_ProfileStart, DateTime.MinValue))
				{
					Core.m_ProfileTime = TimeSpan.op_Addition(Core.m_ProfileTime, DateTime.op_Subtraction(DateTime.Now, Core.m_ProfileStart));
 
				}
				Core.m_ProfileStart = (Core.m_Profiling ? DateTime.Now : DateTime.MinValue);
			}
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		static void Main(string[] args)
		{
			bool debugMode;
			for (int argNum=0; argNum<args.Length; argNum++) 
			{
				if (Insensitive.Equals(args[argNum],"-debug"))
				{
					debugMode = true;
				}
				else if (Insensitive.Equals(args[argNum],"-service"))
				{
					Core.m_Service = true;
				}
				else if (Insensitive.Equals(args[argNum],"-profile"))
				{
					Core.Profiling = true;
				}

				try
				{
					if (Core.m_Service)
					{
						if (!Directory.Exists("logs"))
						{
							Directory.CreateDirectory("logs");
						}
						TextWriter[] outWriters = new TextWriter[2];
						outWriters[0] = Console.Out;
						outWriters[1] = new FileLogger("logs/console.log");
						Core.m_MultiConOut = new MultiTextWriter(outWriters);
						Console.SetOut(new MultiTextWriter(outWriters));
					}
					else
					{
						TextWriter[] outWriter = new TextWriter[1];
						outWriter[0] = Console.Out;
						Core.m_MultiConOut = new MultiTextWriter(outWriter);
						Console.SetOut(new MultiTextWriter(outWriter));
					}
				}
				catch (Exception ignored)
				{
				}

				Core.m_Thread = Thread.CurrentThread;
				Core.m_Process = Process.GetCurrentProcess;
				Core.m_Assembly = Assembly.GetEntryAssembly;
				if (Core.m_Thread != null) 
				{
					Core.m_Thread.Name = "Core Thread";
				}
				if (Core.BaseDirectory.Length > 0)
				{
					Directory.SetCurrentDirectory(Core.BaseDirectory);
				}
				Timer.TimerThread objTimerThread = new Timer.TimerThread();
				Core.timerThread = new Thread(new ThreadStart(objTimerThread.TimerMain));
				Core.timerThread.Name = "Timer Thread";

				while (!ScriptCompiler.Compile(debugMode))
				{
					Console.WriteLine("Scripts: One or more scripts failed to compile or no script files were found.");
					Console.WriteLine(" - Press return to exit, or R to try again.");
					string tryAgainPrompt = Console.ReadLine();
					if ((tryAgainPrompt == null) || (tryAgainPrompt.ToLower() != "r"))
					{
						return;
					}
				}
				// WRITEOUT
			}
			//
			// TODO: Add code to start application here
			//
		}
	}
}
